package com.liquidnet.service.chime.service.impl;

import com.liquidnet.commons.lang.util.BeanUtil;
import com.liquidnet.commons.lang.util.StringUtil;
import com.liquidnet.service.chime.dto.ChimeUserInfoDto;
import com.liquidnet.service.chime.dto.ChimeUserListQueryReqDto;
import com.liquidnet.service.chime.dto.NewPageResult;
import com.liquidnet.service.chime.service.IChimePerformanceService;
import com.liquidnet.service.chime.utils.ObjectUtil;
import com.liquidnet.service.chime.vo.mongo.ChimeUserInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author AnJiabin <anjiabin@zhengzai.tv>
 * @version V1.0
 * @Description: TODO
 * @class: ChimePerformanceServiceImpl
 * @Package com.liquidnet.service.chime.service.impl
 * @Copyright: LightNet @ Copyright (c) 2021
 * @date 2021/9/9 12:04
 */
@Slf4j
@Service
public class ChimePerformanceServiceImpl implements IChimePerformanceService {
    @Autowired
    private MongoTemplate mongoTemplate;
    /**
     * 用户关联查询
     * @param reqDto
     * @return
     */
    @Override
    public NewPageResult<ChimeUserInfoDto> getUserListByCon(ChimeUserListQueryReqDto reqDto) {
        NewPageResult<ChimeUserInfoDto> pagedResult = ObjectUtil.getChimeUserInfoDtoPagedResult();
        try {
            int pageNum = reqDto.getPageNum() - 1;
            int pageSize = reqDto.getPageSize();
            //初始化分页
            pagedResult.setPageSize(pageSize).setCurrentPage(pageNum+1);
            //获取总数

            Criteria criteria = Criteria.where("joinPerformanceId").is(reqDto.getPerformanceId());
            if(StringUtil.isNotEmpty(reqDto.getTags())){
                criteria = criteria.and("userTagsVoList.tagCode").in(Arrays.asList(reqDto.getTags().split(",")));
            }
            if(StringUtil.isNotEmpty(reqDto.getSex())){
                criteria = criteria.and("sex").is(reqDto.getSex());
            }
            Query query = Query.query(criteria);
            query.fields().include("_id");
            // 排序

            // 查询总数
            long startTime = System.currentTimeMillis();
//            int count = (int) mongoTemplate.count(query, ChimeUserInfoVo.class, ChimeUserInfoVo.class.getSimpleName());
            int count = 1000;
            long endTime = System.currentTimeMillis();
            log.info("查询标签数据 count:{} 耗时：{}ms",count,endTime-startTime);


            // 每页五个
            Pageable pageable = PageRequest.of(pageNum, pageSize); // get 5 profiles on a page
            query = Query.query(criteria);
            query.with(pageable);

            startTime = System.currentTimeMillis();
            List<ChimeUserInfoVo> chimeUserInfoVoList = mongoTemplate.find(query, ChimeUserInfoVo.class, ChimeUserInfoVo.class.getSimpleName());

            endTime = System.currentTimeMillis();
            log.info("查询标签数据 分页数据 耗时：{}ms",endTime-startTime);


            //处理Vo
            List<ChimeUserInfoDto> dtoList = new ArrayList<>();
            startTime = System.currentTimeMillis();

            for (ChimeUserInfoVo item : chimeUserInfoVoList) {
                ChimeUserInfoDto voData = ChimeUserInfoDto.getNew();
                BeanUtil.copy(item,voData);
                voData.setPassWord("123456");
                dtoList.add(voData);
            }
            endTime = System.currentTimeMillis();
            log.info("dto转换vo 耗时：{}ms",endTime-startTime);
            pagedResult.setList(dtoList).setTotal(count, pageSize);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return pagedResult;
    }
}
